@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -16,6 +16,7 @@
 #define ADHOCCOMMANDPROVIDER_H__
 
 #include "tag.h"
+#include "jid.h"
 
 #include <list>
 #include <map>
@@ -25,18 +26,19 @@ namespace gloox
 {
 
   /**
-   * @brief A virtual interface for an Ad-hoc Command Provider according to JEP-0050.
+   * @brief A virtual interface for an Ad-hoc Command Provider according to XEP-0050.
    *
    * Derived classes can be registered as Command Providers with the Adhoc object.
+   *
    * @author Jakob Schroeter <js@camaya.net>
    */
-  class GLOOX_EXPORT AdhocCommandProvider
+  class GLOOX_API AdhocCommandProvider
   {
     public:
       /**
        * Virtual destructor.
        */
-      virtual ~AdhocCommandProvider() {};
+      virtual ~AdhocCommandProvider() {}
 
       /**
        * This function is called when an Ad-hoc Command needs to be handled.
@@ -44,8 +46,12 @@ namespace gloox
        * handling etc.
        * @param command The name of the command to be executed.
        * @param tag The complete command tag.
+       * @param from The sender of the command request.
+       * @param id The command's id. An opaque string which should be used only as the id of the
+       * iq result or error stanza.
        */
-      virtual void handleAdhocCommand( const std::string& command, Tag *tag ) = 0;
+      virtual void handleAdhocCommand( const std::string& command, Tag *tag, const JID& from,
+                                       const std::string& id ) = 0;
 
   };
 
