@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -10,10 +10,12 @@
   This software is distributed without any warranty.
 */
 
-#ifdef WIN32
-#include "../config.h.win"
+#ifdef _WIN32
+# include "../config.h.win"
+#elif defined( _WIN32_WCE )
+# include "../config.h.win"
 #else
-#include "config.h"
+# include "config.h"
 #endif
 
 #include "client.h"
@@ -21,26 +23,31 @@
 #include "disco.h"
 #include "logsink.h"
 #include "nonsaslauth.h"
-#include "connection.h"
 #include "tag.h"
-#include "stanza.h"
+#include "stanzaextensionfactory.h"
+#include "stanzaextension.h"
+#include "tlsbase.h"
 
-#ifndef WIN32
-#include <unistd.h>
+#if !defined( _WIN32 ) && !defined( _WIN32_WCE )
+# include <unistd.h>
 #endif
 
-#include <iostream>
-#include <sstream>
+#ifndef _WIN32_WCE
+# include <iostream>
+# include <sstream>
+#else
+# include <stdio.h>
+#endif
 
 namespace gloox
 {
 
   Client::Client( const std::string& server )
     : ClientBase( XMLNS_CLIENT, server ),
-      m_rosterManager( 0 ), m_auth( 0 ), m_disco( 0 ),
-      m_resourceBound( false ), m_autoPresence( false ), m_forceNonSasl( false ),
-      m_manageRoster( true ), m_handleDisco( true ), m_doAuth( false ),
-      m_streamFeatures( 0 ), m_priority( -1 )
+      m_rosterManager( 0 ), m_auth( 0 ),
+      m_presence( PresenceAvailable ), m_resourceBound( false ), m_forceNonSasl( false ),
+      m_manageRoster( true ), m_doAuth( false ),
+      m_streamFeatures( 0 ), m_priority( 0 )
   {
     m_jid.setServer( server );
     init();
@@ -48,10 +55,10 @@ namespace gloox
 
   Client::Client( const JID& jid, const std::string& password, int port )
     : ClientBase( XMLNS_CLIENT, password, "", port ),
-      m_rosterManager( 0 ), m_auth( 0 ), m_disco( 0 ),
-      m_resourceBound( false ), m_autoPresence( false ), m_forceNonSasl( false ),
-      m_manageRoster( true ), m_handleDisco( true ), m_doAuth( true ),
-      m_streamFeatures( 0 ), m_priority( -1 )
+      m_rosterManager( 0 ), m_auth( 0 ),
+      m_presence( PresenceAvailable ), m_resourceBound( false ), m_forceNonSasl( false ),
+      m_manageRoster( true ), m_doAuth( true ),
+      m_streamFeatures( 0 ), m_priority( 0 )
   {
     m_jid = jid;
     m_server = m_jid.serverRaw();
@@ -61,10 +68,10 @@ namespace gloox
   Client::Client( const std::string& username, const std::string& password,
                   const std::string& server, const std::string& resource, int port )
     : ClientBase( XMLNS_CLIENT, password, server, port ),
-      m_rosterManager( 0 ), m_auth( 0 ), m_disco( 0 ),
-      m_resourceBound( false ), m_autoPresence( false ), m_forceNonSasl( false ),
-      m_manageRoster( true ), m_handleDisco( true ), m_doAuth( true ),
-      m_streamFeatures( 0 ), m_priority( -1 )
+      m_rosterManager( 0 ), m_auth( 0 ),
+      m_presence( PresenceAvailable ), m_resourceBound( false ), m_forceNonSasl( false ),
+      m_manageRoster( true ), m_doAuth( true ),
+      m_streamFeatures( 0 ), m_priority( 0 )
   {
     m_jid.setUsername( username );
     m_jid.setServer( server );
@@ -75,16 +82,14 @@ namespace gloox
 
   Client::~Client()
   {
-    delete m_disco;
+    removePresenceExtensions();
     delete m_rosterManager;
     delete m_auth;
   }
 
   void Client::init()
   {
-    m_disco = new Disco( this );
-    m_rosterManager = new RosterManager( this, true );
-    m_disco->setVersion( "based on gloox", GLOOX_VERSION );
+    m_rosterManager = new RosterManager( this );
     m_disco->setIdentity( "client", "bot" );
   }
 
@@ -100,50 +105,47 @@ namespace gloox
     {
       m_streamFeatures = getStreamFeatures( stanza );
 
-#ifdef HAVE_TLS
-      if( tls() && hasTls() && !m_connection->isSecure() && ( m_streamFeatures & StreamFeatureStartTls ) )
+      if( m_tls == TLSRequired
+          && ( !m_encryption || !( m_streamFeatures & StreamFeatureStartTls ) ) )
       {
+        logInstance().log( LogLevelError, LogAreaClassClient,
+                    "Client is configured to require TLS but either the server didn't offer TLS or "
+                    "TLS support is not compiled in." );
+        disconnect( ConnTlsNotAvailable );
+      }
+      else if( m_tls > TLSDisabled && m_encryption && !m_encryptionActive
+          && ( m_streamFeatures & StreamFeatureStartTls ) )
+      {
+        notifyStreamEvent( StreamEventEncryption );
         startTls();
-        return true;
       }
-#endif
-
-#ifdef HAVE_ZLIB
-      if( ( m_streamFeatures & StreamFeatureCompressZlib )
-              && m_connection->initCompression( StreamFeatureCompressZlib ) )
-      {
-        negotiateCompression( StreamFeatureCompressZlib );
-        return true;
-      }
-//       if( ( m_streamFeatures & StreamFeatureCompressDclz )
-//               && m_connection->initCompression( StreamFeatureCompressDclz ) )
-//       {
-//         negotiateCompression( StreamFeatureCompressDclz );
-//         return true;
-//       }
-#endif
-
-      if( sasl() )
+      else if( m_sasl )
       {
         if( m_authed )
         {
           if( m_streamFeatures & StreamFeatureBind )
           {
+            notifyStreamEvent( StreamEventResourceBinding );
             bindResource();
           }
         }
         else if( m_doAuth && !username().empty() && !password().empty() )
         {
-          if( m_streamFeatures & StreamFeatureSaslDigestMd5 && !m_forceNonSasl )
+          if( m_streamFeatures & SaslMechDigestMd5 && m_availableSaslMechs & SaslMechDigestMd5
+              && !m_forceNonSasl )
           {
-            startSASL( SaslDigestMd5 );
+            notifyStreamEvent( StreamEventAuthentication );
+            startSASL( SaslMechDigestMd5 );
           }
-          else if( m_streamFeatures & StreamFeatureSaslPlain && !m_forceNonSasl )
+          else if( m_streamFeatures & SaslMechPlain && m_availableSaslMechs & SaslMechPlain
+                   && !m_forceNonSasl )
           {
-            startSASL( SaslPlain );
+            notifyStreamEvent( StreamEventAuthentication );
+            startSASL( SaslMechPlain );
           }
           else if( m_streamFeatures & StreamFeatureIqAuth || m_forceNonSasl )
           {
+            notifyStreamEvent( StreamEventAuthentication );
             nonSaslLogin();
           }
           else
@@ -154,92 +156,93 @@ namespace gloox
           }
         }
         else if( m_doAuth && !m_clientCerts.empty() && !m_clientKey.empty()
-                 && m_streamFeatures & StreamFeatureSaslExternal )
+                 && m_streamFeatures & SaslMechExternal && m_availableSaslMechs & SaslMechExternal )
         {
-          startSASL( SaslExternal );
+          notifyStreamEvent( StreamEventAuthentication );
+          startSASL( SaslMechExternal );
         }
-        else if( m_doAuth && m_streamFeatures & StreamFeatureSaslAnonymous )
+#ifdef _WIN32
+        else if( m_doAuth && m_streamFeatures & SaslMechGssapi && m_availableSaslMechs & SaslMechGssapi )
         {
-          startSASL( SaslAnonymous );
+          notifyStreamEvent( StreamEventAuthentication );
+          startSASL( SaslMechGssapi );
+        }
+#endif
+        else if( m_doAuth && m_streamFeatures & SaslMechAnonymous
+                 && m_availableSaslMechs & SaslMechAnonymous )
+        {
+          notifyStreamEvent( StreamEventAuthentication );
+          startSASL( SaslMechAnonymous );
         }
         else
         {
+          notifyStreamEvent( StreamEventFinished );
           connected();
         }
       }
+      else if( m_compress && m_compression && !m_compressionActive
+          && ( m_streamFeatures & StreamFeatureCompressZlib ) )
+      {
+        notifyStreamEvent( StreamEventCompression );
+        negotiateCompression( StreamFeatureCompressZlib );
+      }
+//       else if( ( m_streamFeatures & StreamFeatureCompressDclz )
+//               && m_connection->initCompression( StreamFeatureCompressDclz ) )
+//       {
+//         negotiateCompression( StreamFeatureCompressDclz );
+//       }
       else if( m_streamFeatures & StreamFeatureIqAuth )
       {
+        notifyStreamEvent( StreamEventAuthentication );
         nonSaslLogin();
       }
       else
       {
         logInstance().log( LogLevelError, LogAreaClassClient,
-                                 "the server doesn't support any auth mechanisms we know about" );
+                           "fallback: the server doesn't support any auth mechanisms we know about" );
         disconnect( ConnNoSupportedAuth );
       }
     }
-#ifdef HAVE_TLS
     else if( ( stanza->name() == "proceed" ) && stanza->hasAttribute( "xmlns", XMLNS_STREAM_TLS ) )
     {
       logInstance().log( LogLevelDebug, LogAreaClassClient, "starting TLS handshake..." );
 
-      if( m_connection->tlsHandshake() )
+      if( m_encryption )
       {
-        if( !notifyOnTLSConnect( m_connection->fetchTLSInfo() ) )
-          disconnect( ConnTlsFailed );
-        else
-        {
-          std::ostringstream oss;
-          if( m_connection->isSecure() )
-          {
-            oss << "connection encryption active";
-            logInstance().log( LogLevelDebug, LogAreaClassClient, oss.str() );
-          }
-          else
-          {
-            oss << "connection not encrypted!";
-            logInstance().log( LogLevelWarning, LogAreaClassClient, oss.str() );
-          }
-
-          header();
-        }
+        m_encryptionActive = true;
+        m_encryption->handshake();
       }
-      else
-        disconnect( ConnTlsFailed );
     }
     else if( ( stanza->name() == "failure" ) && stanza->hasAttribute( "xmlns", XMLNS_STREAM_TLS ) )
     {
-      logInstance().log( LogLevelError, LogAreaClassClient, "TLS handshake failed!" );
+      logInstance().log( LogLevelError, LogAreaClassClient, "TLS handshake failed (server-side)!" );
       disconnect( ConnTlsFailed );
     }
-#endif
-#ifdef HAVE_ZLIB
     else if( ( stanza->name() == "failure" ) && stanza->hasAttribute( "xmlns", XMLNS_COMPRESSION ) )
     {
       logInstance().log( LogLevelError, LogAreaClassClient, "stream compression init failed!" );
-      disconnect( ConnTlsFailed );
+      disconnect( ConnCompressionFailed );
     }
     else if( ( stanza->name() == "compressed" ) && stanza->hasAttribute( "xmlns", XMLNS_COMPRESSION ) )
     {
       logInstance().log( LogLevelDebug, LogAreaClassClient, "stream compression inited" );
-      m_connection->enableCompression();
+      m_compressionActive = true;
       header();
     }
-#endif
     else if( ( stanza->name() == "challenge" ) && stanza->hasAttribute( "xmlns", XMLNS_STREAM_SASL ) )
     {
-      logInstance().log( LogLevelDebug, LogAreaClassClient, "processing sasl challenge" );
+      logInstance().log( LogLevelDebug, LogAreaClassClient, "processing SASL challenge" );
       processSASLChallenge( stanza->cdata() );
     }
     else if( ( stanza->name() == "failure" ) && stanza->hasAttribute( "xmlns", XMLNS_STREAM_SASL ) )
     {
-      logInstance().log( LogLevelError, LogAreaClassClient, "sasl authentication failed!" );
+      logInstance().log( LogLevelError, LogAreaClassClient, "SASL authentication failed!" );
       processSASLError( stanza );
       disconnect( ConnAuthenticationFailed );
     }
     else if( ( stanza->name() == "success" ) && stanza->hasAttribute( "xmlns", XMLNS_STREAM_SASL ) )
     {
-      logInstance().log( LogLevelDebug, LogAreaClassClient, "sasl auth successful" );
+      logInstance().log( LogLevelDebug, LogAreaClassClient, "SASL authentication successful" );
       setAuthed( true );
       header();
     }
@@ -285,9 +288,6 @@ namespace gloox
     if( stanza->hasChild( "register", "xmlns", XMLNS_STREAM_IQREGISTER ) )
       features |= StreamFeatureIqRegister;
 
-    if( stanza->hasChild( "ack", "xmlns", XMLNS_STREAM_ACK ) )
-      features |= StreamFeatureAck;
-
     if( stanza->hasChild( "compression", "xmlns", XMLNS_STREAM_COMPRESS ) )
       features |= getCompressionMethods( stanza->findChild( "compression" ) );
 
@@ -299,19 +299,22 @@ namespace gloox
 
   int Client::getSaslMechs( Tag *tag )
   {
-    int mechs = 0;
+    int mechs = SaslMechNone;
 
     if( tag->hasChildWithCData( "mechanism", "DIGEST-MD5" ) )
-      mechs |= StreamFeatureSaslDigestMd5;
+      mechs |= SaslMechDigestMd5;
 
     if( tag->hasChildWithCData( "mechanism", "PLAIN" ) )
-      mechs |= StreamFeatureSaslPlain;
+      mechs |= SaslMechPlain;
 
     if( tag->hasChildWithCData( "mechanism", "ANONYMOUS" ) )
-      mechs |= StreamFeatureSaslAnonymous;
+      mechs |= SaslMechAnonymous;
 
     if( tag->hasChildWithCData( "mechanism", "EXTERNAL" ) )
-      mechs |= StreamFeatureSaslExternal;
+      mechs |= SaslMechExternal;
+
+    if( tag->hasChildWithCData( "mechanism", "GSSAPI" ) )
+      mechs |= SaslMechGssapi;
 
     return mechs;
   }
@@ -390,6 +393,7 @@ namespace gloox
 
   void Client::createSession()
   {
+    notifyStreamEvent( StreamEventSessionCreation );
     Tag *iq = new Tag( "iq" );
     iq->addAttribute( "type", "set" );
     iq->addAttribute( "id", "session" );
@@ -449,11 +453,33 @@ namespace gloox
     send( t );
   }
 
-  void Client::disableDisco()
+  void Client::addPresenceExtension( StanzaExtension *se )
   {
-    m_handleDisco = false;
-    delete m_disco;
-    m_disco = 0;
+    m_presenceExtensions.push_back( se );
+  }
+
+  void Client::removePresenceExtensions()
+  {
+    StanzaExtensionList::iterator it = m_presenceExtensions.begin();
+    for( ; it != m_presenceExtensions.end(); ++it )
+    {
+      delete (*it);
+    }
+  }
+
+  void Client::setPresence( Presence presence, int priority, const std::string& status )
+  {
+    m_presence = presence;
+    m_status = status;
+
+    if( priority < -128 )
+      m_priority = -128;
+    if( priority > 127 )
+      m_priority = 127;
+    else
+      m_priority = priority;
+
+    sendPresence();
   }
 
   void Client::disableRoster()
@@ -465,38 +491,36 @@ namespace gloox
 
   void Client::nonSaslLogin()
   {
-    m_auth = new NonSaslAuth( this, m_sid );
-    m_auth->doAuth();
+    if( !m_auth )
+      m_auth = new NonSaslAuth( this );
+    m_auth->doAuth( m_sid );
   }
 
-  void Client::sendInitialPresence()
+  void Client::sendPresence()
   {
-    Tag *p = new Tag( "presence" );
-    std::ostringstream oss;
-    oss << m_priority;
-    new Tag( p, "priority", oss.str() );
+    if( m_presence != PresenceUnknown &&
+        state() >= StateConnected )
+    {
+      JID jid;
+      Stanza *p = Stanza::createPresenceStanza( jid, m_status, m_presence );
+#ifdef _WIN32_WCE
+      char tmp[5];
+      tmp[4] = '\0';
+      sprintf( tmp, "%s", m_priority );
+      new Tag( p, "priority", tmp );
+#else
+      std::ostringstream oss;
+      oss << m_priority;
+      new Tag( p, "priority", oss.str() );
+#endif
+      StanzaExtensionList::const_iterator it = m_presenceExtensions.begin();
+      for( ; it != m_presenceExtensions.end(); ++it )
+      {
+        p->addChild( (*it)->tag() );
+      }
 
-    send( p );
-  }
-
-  void Client::setInitialPriority( int priority )
-  {
-    if( priority < -128 )
-      priority = -128;
-    if( priority > 127 )
-      priority = 127;
-
-    m_priority = priority;
-  }
-
-  RosterManager* Client::rosterManager()
-  {
-    return m_rosterManager;
-  }
-
-  Disco* Client::disco()
-  {
-    return m_disco;
+      send( p );
+    }
   }
 
   void Client::connected()
@@ -504,18 +528,25 @@ namespace gloox
     if( m_authed )
     {
       if( m_manageRoster )
+      {
+        notifyStreamEvent( StreamEventRoster );
         m_rosterManager->fill();
+      }
       else
         rosterFilled();
     }
-
-    notifyOnConnect();
+    else
+    {
+      notifyStreamEvent( StreamEventFinished );
+      notifyOnConnect();
+    }
   }
 
   void Client::rosterFilled()
   {
-    if( m_autoPresence )
-      sendInitialPresence();
+    sendPresence();
+    notifyStreamEvent( StreamEventFinished );
+    notifyOnConnect();
   }
 
   void Client::disconnect()
@@ -531,4 +562,11 @@ namespace gloox
     ClientBase::disconnect( reason );
   }
 
+  void Client::cleanup()
+  {
+    m_authed = false;
+    m_resourceBound = false;
+    m_streamFeatures = 0;
+  }
+
 }
