@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -17,50 +17,44 @@
 #include "disco.h"
 #include "stanza.h"
 #include "prep.h"
+#include "sha.h"
 
-#include <iksemel.h>
+#include <cstdlib>
 
 namespace gloox
 {
 
   Component::Component( const std::string& ns, const std::string& server,
                         const std::string& component, const std::string& password, int port )
-    : ClientBase( ns, password, server, port ),
-      m_disco( 0 )
+    : ClientBase( ns, password, server, port )
   {
     m_jid.setServer( component );
-    m_disco = new Disco( this );
-    m_disco->setVersion( "based on gloox", GLOOX_VERSION );
     m_disco->setIdentity( "component", "generic" );
   }
 
-  Component::~Component()
-  {
-    delete m_disco;
-  }
-
   void Component::handleStartNode()
   {
     if( m_sid.empty() )
       return;
 
-    const std::string data = m_sid + m_password;
-    char *hash = (char*)calloc( 41, sizeof( char ) );
-    iks_sha( data.c_str(), hash );
+    notifyStreamEvent( StreamEventAuthentication );
 
-    Tag *h = new Tag( "handshake", hash );
+    SHA sha;
+    sha.feed( m_sid + m_password );
+    sha.finalize();
+
+    Tag *h = new Tag( "handshake", sha.hex() );
     send( h );
-
-    free( hash );
   }
 
   bool Component::handleNormalNode( Stanza *stanza )
   {
-    if( stanza->name() == "handshake" )
-      notifyOnConnect();
-    else
+    if( stanza->name() != "handshake" )
       return false;
 
+    notifyStreamEvent( StreamEventFinished );
+    notifyOnConnect();
+
     return true;
   }
 
