@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -22,12 +22,11 @@
 namespace gloox
 {
 
-  class Disco;
-
   /**
    * @brief This is an implementation of a basic jabber Component.
    *
-   * It's using JEP-0114 (Jabber Component Protocol) to authenticate with a server.
+   * It's using XEP-0114 (Jabber Component Protocol) to authenticate with a server.
+   *
    * @author Jakob Schroeter <js@camaya.net>
    * @since 0.3
    */
@@ -37,7 +36,7 @@ namespace gloox
       /**
        * Constructs a new Component.
        * @param ns The namespace that qualifies the stream. Either @b jabber:component:accept or
-       * @b jabber:component:connect.
+       * @b jabber:component:connect. See XEP-0114 for details.
        * @param server The server to connect to.
        * @param component The component's hostname. FQDN.
        * @param password The component's password.
@@ -50,36 +49,22 @@ namespace gloox
       /**
        * Virtual Destructor.
        */
-      virtual ~Component();
-
-      // reimplemented from ClientBase
-      virtual const std::string username() const { return ""; };
-
-      // reimplemented from ClientBase
-      virtual const std::string password() const { return ""; };
-
-      /**
-       * Gives access to the component's Disco object.
-       * @return A pointer to the Disco object.
-       */
-      Disco* disco() const { return m_disco; };
+      virtual ~Component() {}
 
       /**
        * Disconnects from the server.
        */
-      void disconnect();
+      void disconnect() { ClientBase::disconnect( ConnUserDisconnected ); }
 
     protected:
       virtual void handleStartNode();
       virtual bool handleNormalNode( Stanza *stanza );
-      virtual bool checkStreamVersion( const std::string& /*version*/ ) { return true; };
-      virtual void disconnect( ConnectionError reason );
+      virtual bool checkStreamVersion( const std::string& /*version*/ ) { return true; }
 
     private:
       // reimplemented from ClientBase
-      virtual void rosterFilled() {};
+      virtual void rosterFilled() {}
 
-      Disco *m_disco;
   };
 
 }
