@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -13,6 +13,8 @@
 
 #include "dataform.h"
 #include "dataformfield.h"
+#include "dataformreported.h"
+#include "dataformitem.h"
 #include "tag.h"
 
 namespace gloox
@@ -23,24 +25,43 @@ namespace gloox
   {
   }
 
+  DataForm::DataForm( DataFormType type, const std::string& title )
+    : m_type( type ), m_title( title )
+  {
+  }
+
   DataForm::DataForm( Tag *tag )
-    : m_type( FORM_TYPE_INVALID )
+    : m_type( FormTypeInvalid )
+  {
+    parse( tag );
+  }
+
+  DataForm::DataForm()
+  : m_type( FormTypeInvalid )
+  {
+  }
+
+  DataForm::~DataForm()
+  {
+  }
+
+  bool DataForm::parse( Tag *tag )
   {
     if( !tag || !tag->hasAttribute( "xmlns", XMLNS_X_DATA ) || tag->name() != "x" )
-      return;
+      return false;
 
     if( tag->hasAttribute( "type", "form" ) )
-      m_type = FORM_TYPE_FORM;
+      m_type = FormTypeForm;
     else if( tag->hasAttribute( "type", "submit" ) )
-      m_type = FORM_TYPE_SUBMIT;
+      m_type = FormTypeSubmit;
     else if( tag->hasAttribute( "type", "cancel" ) )
-      m_type = FORM_TYPE_CANCEL;
+      m_type = FormTypeCancel;
     else if( tag->hasAttribute( "type", "result" ) )
-      m_type = FORM_TYPE_RESULT;
+      m_type = FormTypeResult;
     else
-      return;
+      return false;
 
-    Tag::TagList l = tag->children();
+    const Tag::TagList& l = tag->children();
     Tag::TagList::const_iterator it = l.begin();
     for( ; it != l.end(); ++it )
     {
@@ -50,19 +71,27 @@ namespace gloox
         m_instructions.push_back( (*it)->cdata() );
       else if( (*it)->name() == "field" )
       {
-        DataFormField f( (*it) );
+        DataFormField *f = new DataFormField( (*it) );
         m_fields.push_back( f );
       }
+      else if( (*it)->name() == "reported" )
+      {
+        DataFormReported *r = new DataFormReported( (*it) );
+        m_fields.push_back( r );
+      }
+      else if( (*it)->name() == "item" )
+      {
+        DataFormItem *i = new DataFormItem( (*it) );
+        m_fields.push_back( i );
+      }
     }
+
+    return true;
   }
 
-  DataForm::~DataForm()
+  Tag* DataForm::tag() const
   {
-  }
-
-  const Tag* DataForm::tag() const
-  {
-    if( m_type == FORM_TYPE_INVALID )
+    if( m_type == FormTypeInvalid )
       return 0;
 
     Tag *x = new Tag( "x" );
@@ -77,21 +106,35 @@ namespace gloox
     FieldList::const_iterator it = m_fields.begin();
     for( ; it != m_fields.end(); ++it )
     {
-      x->addChild( (*it).tag() );
+      DataFormItem *i = dynamic_cast<DataFormItem*>( (*it) );
+      if( i )
+      {
+        x->addChild( i->tag() );
+        continue;
+      }
+
+      DataFormReported *r = dynamic_cast<DataFormReported*>( (*it) );
+      if( r )
+      {
+        x->addChild( r->tag() );
+        continue;
+      }
+
+      x->addChild( (*it)->tag() );
     }
 
     switch( m_type )
     {
-      case FORM_TYPE_FORM:
+      case FormTypeForm:
         x->addAttribute( "type", "form" );
         break;
-      case FORM_TYPE_SUBMIT:
+      case FormTypeSubmit:
         x->addAttribute( "type", "submit" );
         break;
-      case FORM_TYPE_CANCEL:
+      case FormTypeCancel:
         x->addAttribute( "type", "cancel" );
         break;
-      case FORM_TYPE_RESULT:
+      case FormTypeResult:
         x->addAttribute( "type", "result" );
         break;
       default:
