@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -12,6 +12,8 @@
 
 
 #include "dataformbase.h"
+#include "dataformitem.h"
+#include "dataformreported.h"
 
 
 namespace gloox
@@ -23,6 +25,12 @@ namespace gloox
 
   DataFormBase::~DataFormBase()
   {
+    FieldList::iterator it = m_fields.begin();
+    for( ; it != m_fields.end(); ++it )
+    {
+      delete (*it);
+      (*it) = 0;
+    }
   }
 
   bool DataFormBase::hasField( const std::string& field )
@@ -30,23 +38,23 @@ namespace gloox
     FieldList::const_iterator it = m_fields.begin();
     for( ; it != m_fields.end(); ++it )
     {
-      if( (*it).name() == field )
+      if( (*it)->name() == field )
         return true;
     }
 
     return false;
   }
 
-  DataFormField DataFormBase::field( const std::string& field )
+  DataFormField* DataFormBase::field( const std::string& field )
   {
     FieldList::const_iterator it = m_fields.begin();
     for( ; it != m_fields.end(); ++it )
     {
-      if( (*it).name() == field )
+      if( (*it)->name() == field )
         return (*it);
     }
 
-    return DataFormField();
+    return 0;
   }
 
 }
