@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -21,26 +21,37 @@ namespace gloox
 {
 
   /**
-   * @brief An abstraction of an &lt;item&gt; element in a JEP-0004 Data Form of type result.
+   * @brief An abstraction of an &lt;item&gt; element in a XEP-0004 Data Form of type result.
    *
-   * There are some constraints regarding usage of this element you should be aware of. Check JEP-0004
+   * There are some constraints regarding usage of this element you should be aware of. Check XEP-0004
    * section 3.4. This class does not enforce correct usage at this point.
    *
    * @author Jakob Schroeter <js@camaya.net>
    * @since 0.7
    */
-  class GLOOX_EXPORT DataFormItem : public DataFormBase, DataFormField
+  class GLOOX_API DataFormItem : public DataFormBase, public DataFormField
   {
     public:
       /**
        * Creates an empty 'item' element you can add fields to.
        */
-      DataFormItem() : DataFormField( FIELD_TYPE_ITEM ) {};
+      DataFormItem();
+
+      /**
+       * Creates a 'item' element and fills it with the 'field' elements contained in the given Tag.
+       * The Tag's root element must be a 'item' element. Its child element should be 'field' elements.
+       * @param tag The tag to read the 'field' elements from.
+       * @since 0.8.5
+       */
+      DataFormItem( Tag *tag );
 
       /**
        * Virtual destructor.
        */
-      virtual ~DataFormItem() {};
+      virtual ~DataFormItem();
+
+      // reimplemented from DataFormField
+      virtual Tag* tag() const;
 
   };
 
