@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -22,9 +22,10 @@ namespace gloox
 {
 
   /**
-   * @brief A virtual interface that enables objects to receive Service Discovery (JEP-0030) events.
+   * @brief A virtual interface that enables objects to receive Service Discovery (XEP-0030) events.
+   *
+   * A class implementing this interface can receive the results of sent disco queries.
    *
-   * A class implementing this interface can receives the results of sent disco queries.
    * @author Jakob Schroeter <js@camaya.net>
    */
   class GLOOX_API DiscoHandler
@@ -33,7 +34,7 @@ namespace gloox
       /**
        * Virtual Destructor.
        */
-      virtual ~DiscoHandler() {};
+      virtual ~DiscoHandler() {}
 
       /**
        * Reimplement this function if you want to be notified about the result
@@ -68,7 +69,7 @@ namespace gloox
        * @return Returns @b true if the stanza was handled and answered, @b false otherwise.
        * @todo Replace the stanza with decoded values.
        */
-      virtual bool handleDiscoSet( Stanza *stanza ) { (void) stanza; return false; };
+      virtual bool handleDiscoSet( Stanza *stanza ) { (void) stanza; return false; }
 
   };
 
