@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -11,30 +11,44 @@
 */
 
 
-#ifdef WIN32
-#include "../config.h.win"
+#ifdef _WIN32
+# include "../config.h.win"
+#elif defined( _WIN32_WCE )
+# include "../config.h.win"
 #else
-#include "config.h"
+# include "config.h"
 #endif
 
+#include "gloox.h"
 #include "dns.h"
 
-#include <sys/types.h>
-
-#ifndef WIN32
-#include <netinet/in.h>
-#include <resolv.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <netdb.h>
-#include <unistd.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#else
-#include <winsock.h>
+#ifndef _WIN32_WCE
+# include <sys/types.h>
+# include <sstream>
 #endif
 
-#include <sstream>
+#include <stdio.h>
+
+#if !defined( _WIN32 ) && !defined( _WIN32_WCE )
+# include <netinet/in.h>
+# include <arpa/nameser.h>
+# include <resolv.h>
+# include <netdb.h>
+# include <arpa/inet.h>
+# include <sys/socket.h>
+# include <sys/un.h>
+# include <unistd.h>
+#endif
+
+#ifdef _WIN32
+# include <winsock.h>
+#elif defined( _WIN32_WCE )
+# include <winsock2.h>
+#endif
+
+#ifdef HAVE_WINDNS_H
+# include <windns.h>
+#endif
 
 #define SRV_COST    (RRFIXEDSZ+0)
 #define SRV_WEIGHT  (RRFIXEDSZ+2)
@@ -43,15 +57,20 @@
 #define SRV_FIXEDSZ (RRFIXEDSZ+6)
 
 #ifndef T_SRV
-#define T_SRV 33
+# define T_SRV 33
+#endif
+
+// mingw
+#ifndef DNS_TYPE_SRV
+# define DNS_TYPE_SRV 33
 #endif
 
 #ifndef C_IN
-#define C_IN 1
+# define C_IN 1
 #endif
 
 #ifndef INVALID_SOCKET
-#define INVALID_SOCKET -1
+# define INVALID_SOCKET -1
 #endif
 
 #define XMPP_PORT 5222
@@ -59,25 +78,9 @@
 namespace gloox
 {
 
-#if !defined( HAVE_RES_QUERYDOMAIN ) || !defined( HAVE_DN_SKIPNAME ) || !defined( HAVE_RES_QUERY )
-  int DNS::connect( const std::string& domain, const LogSink& logInstance )
-  {
-    logInstance.log( LogLevelWarning, LogAreaClassDns,
-                     "note: gloox does not support SRV records on this platform." );
-
-    return DNS::connect( domain, XMPP_PORT, logInstance );
-  }
-#else
-  DNS::HostMap DNS::resolve( const std::string& domain )
-  {
-    std::string service = "xmpp-client";
-    std::string proto = "tcp";
-
-    return resolve( service, proto, domain );
-  }
-
+#if defined( HAVE_RES_QUERYDOMAIN ) && defined( HAVE_DN_SKIPNAME ) && defined( HAVE_RES_QUERY )
   DNS::HostMap DNS::resolve( const std::string& service, const std::string& proto,
-                             const std::string& domain )
+                             const std::string& domain, const LogSink& logInstance )
   {
     buffer srvbuf;
     bool error = false;
@@ -91,7 +94,7 @@ namespace gloox
       srvbuf.len = res_query( dname.c_str(), C_IN, T_SRV, srvbuf.buf, NS_PACKETSZ );
 
     if( srvbuf.len < 0 )
-      return defaultHostMap( service, proto, domain );
+      return defaultHostMap( domain, logInstance );
 
     HEADER* hdr = (HEADER*)srvbuf.buf;
     unsigned char* here = srvbuf.buf + NS_HFIXEDSZ;
@@ -128,7 +131,7 @@ namespace gloox
 
     if( error )
     {
-      return defaultHostMap( service, proto, domain );
+      return defaultHostMap( domain, logInstance );
     }
 
     // (q)sort here
@@ -147,109 +150,121 @@ namespace gloox
     return servers;
   }
 
-  DNS::HostMap DNS::defaultHostMap( const std::string& service, const std::string& proto,
-                               const std::string& domain )
+#elif defined( _WIN32 ) && defined( HAVE_WINDNS_H )
+  DNS::HostMap DNS::resolve( const std::string& service, const std::string& proto,
+                             const std::string& domain, const LogSink& logInstance )
   {
-    HostMap server;
-    struct servent *servent;
+    const std::string dname = "_" +  service + "._" + proto + "." + domain;
+    bool error = false;
 
-    if( ( servent = getservbyname( service.c_str(), proto.c_str() ) ) == 0 )
+    DNS::HostMap servers;
+    DNS_RECORD *pRecord;
+    if( DnsQuery( dname.c_str(), DNS_TYPE_SRV, DNS_QUERY_STANDARD, NULL, &pRecord, NULL ) == ERROR_SUCCESS )
     {
-      server[domain] = 0;
-      return server;
+      DNS_RECORD *pRec = pRecord;
+      do
+      {
+        if( pRec->wType == DNS_TYPE_SRV )
+        {
+          servers[pRec->Data.SRV.pNameTarget] = pRec->Data.SRV.wPort;
+        }
+        pRec = pRec->pNext;
+      }
+      while( pRec != NULL );
+      DnsRecordListFree( pRecord, DnsFreeRecordList );
+    }
+    else
+      error = true;
+
+    if( error || !servers.size() )
+    {
+      servers = defaultHostMap( domain, logInstance );
     }
 
+    return servers;
+  }
+
+#else
+  DNS::HostMap DNS::resolve( const std::string& /*service*/, const std::string& /*proto*/,
+                             const std::string& domain, const LogSink& logInstance )
+  {
+    logInstance.log( LogLevelWarning, LogAreaClassDns,
+                    "notice: gloox does not support SRV records on this platform. Using A records instead." );
+    return defaultHostMap( domain, logInstance );
+  }
+#endif
+
+  DNS::HostMap DNS::defaultHostMap( const std::string& domain, const LogSink& logInstance )
+  {
+    HostMap server;
+
+    logInstance.log( LogLevelWarning, LogAreaClassDns, "notice: no SRV record found for " + domain
+                                                       + ", using default port." );
+
     if( !domain.empty() )
-      server[domain] = ntohs( servent->s_port );
+      server[domain] = XMPP_PORT;
 
     return server;
   }
 
   int DNS::connect( const std::string& domain, const LogSink& logInstance )
   {
-    HostMap hosts = resolve( domain );
+    HostMap hosts = resolve( domain, logInstance );
     if( hosts.size() == 0 )
-      return -DNS_NO_HOSTS_FOUND;
+      return -ConnDnsError;
 
-    struct protoent* prot;
-    if( ( prot = getprotobyname( "tcp" ) ) == 0)
-      return -DNS_COULD_NOT_RESOLVE;
-
-    int fd;
-    if( ( fd = socket( PF_INET, SOCK_STREAM, prot->p_proto ) ) == -1 )
-      return -DNS_COULD_NOT_RESOLVE;
-
-    struct hostent *h;
-    struct sockaddr_in target;
-    target.sin_family = AF_INET;
-    int ret = 0;
     HostMap::const_iterator it = hosts.begin();
     for( ; it != hosts.end(); ++it )
     {
-      int port;
-      if( (*it).second == 0 )
-        port = XMPP_PORT;
-      else
-        port = (*it).second;
-
-      target.sin_port = htons( port );
-      if( ( h = gethostbyname( (*it).first.c_str() ) ) == 0 )
-      {
-        ret = -DNS_COULD_NOT_RESOLVE;
-        continue;
-      }
-
-      in_addr *addr = (in_addr*)malloc( sizeof( in_addr ) );
-      memcpy( addr, h->h_addr, sizeof( in_addr ) );
-      char *tmp = inet_ntoa( *addr );
-      free( addr );
-      std::ostringstream oss;
-      oss << "resolved " << (*it).first.c_str() <<  " to: " << tmp << ":" << port;
-      logInstance.log( LogLevelDebug, LogAreaClassDns, oss.str() );
-
-      if( inet_aton( tmp, &(target.sin_addr) ) == 0 )
-        continue;
-
-      memset( target.sin_zero, '\0', 8 );
-      if( ::connect( fd, (struct sockaddr *)&target, sizeof( struct sockaddr ) ) == 0 )
+      int fd = DNS::connect( (*it).first, (*it).second, logInstance );
+      if( fd >= 0 )
         return fd;
-
-      close( fd );
     }
-    if( ret )
-      return ret;
 
-    return -DNS_COULD_NOT_CONNECT;
+    return -ConnConnectionRefused;
   }
-#endif
 
-  int DNS::connect( const std::string& domain, int port, const LogSink& logInstance )
+  int DNS::getSocket()
   {
-#ifdef WIN32
+#ifdef _WIN32
     WSADATA wsaData;
     if( WSAStartup( MAKEWORD( 1, 1 ), &wsaData ) != 0 )
-      return -DNS_COULD_NOT_RESOLVE;
+      return -ConnDnsError;
 #endif
 
     struct protoent* prot;
     if( ( prot = getprotobyname( "tcp" ) ) == 0 )
     {
       cleanup();
-      return -DNS_COULD_NOT_RESOLVE;
+      return -ConnDnsError;
     }
 
     int fd;
     if( ( fd = socket( PF_INET, SOCK_STREAM, prot->p_proto ) ) == -1 )
     {
       cleanup();
-      return -DNS_COULD_NOT_CONNECT;
+      return -ConnConnectionRefused;
     }
 
+#ifdef HAVE_SETSOCKOPT
+    int val = 1;
+    setsockopt( fd, SOL_SOCKET, SO_REUSEADDR, (char*)&val, sizeof( val ) );
+#endif
+
+    return fd;
+  }
+
+  int DNS::connect( const std::string& domain, unsigned short port, const LogSink& logInstance )
+  {
+    int fd = getSocket();
+    if( fd < 0 )
+      return fd;
+
     struct hostent *h;
     if( ( h = gethostbyname( domain.c_str() ) ) == 0 )
     {
       cleanup();
-      return -DNS_COULD_NOT_RESOLVE;
+      return -ConnDnsError;
     }
 
     struct sockaddr_in target;
@@ -259,34 +274,52 @@ namespace gloox
     if( h->h_length != sizeof( struct in_addr ) )
     {
       cleanup();
-      return -DNS_COULD_NOT_RESOLVE;
+      return -ConnDnsError;
     }
     else
     {
       memcpy( &target.sin_addr, h->h_addr, sizeof( struct in_addr ) );
     }
 
+#ifndef _WIN32_WCE
     std::ostringstream oss;
-    oss << "resolved " << domain.c_str() << " to: " << inet_ntoa( target.sin_addr );
-    logInstance.log( LogLevelDebug, LogAreaClassDns, oss.str() );
+#endif
 
     memset( target.sin_zero, '\0', 8 );
     if( ::connect( fd, (struct sockaddr *)&target, sizeof( struct sockaddr ) ) == 0 )
+    {
+#ifndef _WIN32_WCE
+      oss << "connecting to " << domain.c_str()
+          << " (" << inet_ntoa( target.sin_addr ) << ":" << port << ")";
+      logInstance.log( LogLevelDebug, LogAreaClassDns, oss.str() );
+#endif
       return fd;
+    }
 
-#ifndef WIN32
+#ifndef _WIN32_WCE
+    oss << "connection to " << domain.c_str()
+         << " (" << inet_ntoa( target.sin_addr ) << ":" << port << ") failed";
+    logInstance.log( LogLevelDebug, LogAreaClassDns, oss.str() );
+#endif
+
+    closeSocket( fd );
+    return -ConnConnectionRefused;
+  }
+
+  void DNS::closeSocket( int fd )
+  {
+#ifndef _WIN32
     close( fd );
 #else
     closesocket( fd );
-    cleanup();
 #endif
-    return -DNS_COULD_NOT_CONNECT;
   }
 
   void DNS::cleanup()
   {
-#ifdef WIN32
+#ifdef _WIN32
     WSACleanup();
 #endif
   }
+
 }
