@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -15,36 +15,39 @@
 #ifndef LOGHANDLER_H__
 #define LOGHANDLER_H__
 
+#include "gloox.h"
+
 #include <string>
 
 namespace gloox
 {
 
   /**
-   * @brief A virtual interface which can be reimplemented to receive data sent and received over
-   * the network.
+   * @brief A virtual interface which can be reimplemented to receive debug and log messages.
+   *
+   * @ref handleLog() is called for log messages.
    *
-   * Upon an incoming packet @ref handleLog() is be called.
    * @author Jakob Schroeter <js@camaya.net>
    * @since 0.5
    */
-  class GLOOX_EXPORT LogHandler
+  class GLOOX_API LogHandler
   {
     public:
       /**
        * Virtual Destructor.
        */
-      virtual ~LogHandler() {};
+      virtual ~LogHandler() {}
 
       /**
        * Reimplement this function if you want to receive the chunks of the conversation
        * between gloox and server.
-       * @param xml The sent or received XML.
-       * @param incoming @b True if the string was received, @b false if it was sent.
+       * @param level The log message's severity.
+       * @param area The log message's origin.
+       * @param message The log message.
        */
-      virtual void handleLog( const std::string& xml, bool incoming ) = 0;
+      virtual void handleLog( LogLevel level, LogArea area, const std::string& message ) = 0;
   };
 
 }
 
-#endif // PRESENCEHANDLER_H__
+#endif // LOGHANDLER_H__
