@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -14,12 +14,12 @@
 #ifndef GLOOX_MACROS_H__
 #define GLOOX_MACROS_H__
 
-#ifdef WIN32
-#pragma warning( disable:4251 )
-#pragma warning( disable:4786 )
+#if defined( _MSC_VER ) || defined( _WIN32_WCE )
+#  pragma warning( disable:4251 )
+#  pragma warning( disable:4786 )
 #endif
 
-#ifdef WIN32
+#ifdef _WIN32
 #  ifdef GLOOX_EXPORTS
 #    define GLOOX_API __declspec( dllexport )
 #  else
@@ -28,17 +28,21 @@
 #    endif
 #  endif
 #endif
+
 #ifndef GLOOX_API
 #  define GLOOX_API
 #endif
 
 
 #if defined( __GNUC__ ) && ( __GNUC__ - 0 > 3 || ( __GNUC__ - 0 == 3 && __GNUC_MINOR__ - 0 >= 2 ) )
-#define GLOOX_DEPRECATED __attribute__ ( (deprecated) )
+#  define GLOOX_DEPRECATED __attribute__ ( (deprecated) )
+#  define GLOOX_DEPRECATED_CTOR explicit GLOOX_DEPRECATED
 #elif defined( _MSC_VER ) && ( _MSC_VER >= 1300 )
-#define GLOOX_DEPRECATED __declspec( deprecated )
+#  define GLOOX_DEPRECATED __declspec( deprecated )
+#  define GLOOX_DEPRECATED_CTOR explicit GLOOX_DEPRECATED
 #else
-#define GLOOX_DEPRECATED
+#  define GLOOX_DEPRECATED
+#  define GLOOX_DEPRECATED_CTOR
 #endif
 
 
