@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -14,6 +14,8 @@
 #ifndef MESSAGEEVENTHANDLER_H__
 #define MESSAGEEVENTHANDLER_H__
 
+#include "gloox.h"
+
 namespace gloox
 {
 
@@ -21,7 +23,7 @@ namespace gloox
 
   /**
    * @brief A virtual interface that enables an object to be notified about
-   * Message Events (JEP-0022).
+   * Message Events (XEP-0022).
    *
    * @author Jakob Schroeter <js@camaya.net>
    * @since 0.8
@@ -32,7 +34,7 @@ namespace gloox
       /**
        * Virtual Destructor.
        */
-      virtual ~MessageEventHandler() {};
+      virtual ~MessageEventHandler() {}
 
       /**
        * Notifies the MessageEventHandler that an event has been raised by the remote
