@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -13,16 +13,15 @@
 
 #include "nonsaslauth.h"
 #include "client.h"
+#include "sha.h"
 
 #include <string>
 
-#include <iksemel.h>
-
 namespace gloox
 {
 
-  NonSaslAuth::NonSaslAuth( Client *parent, const std::string& sid )
-    : m_parent( parent ), m_sid( sid )
+  NonSaslAuth::NonSaslAuth( Client *parent )
+    : m_parent( parent )
   {
     if( m_parent )
       m_parent->registerIqHandler( this, XMLNS_AUTH );
@@ -34,9 +33,10 @@ namespace gloox
       m_parent->removeIqHandler( XMLNS_AUTH );
   }
 
-  void NonSaslAuth::doAuth()
+  void NonSaslAuth::doAuth( const std::string& sid )
   {
-    std::string id = m_parent->getID();
+    m_sid = sid;
+    const std::string& id = m_parent->getID();
 
     Tag *iq = new Tag( "iq" );
     iq->addAttribute( "to", m_parent->jid().server() );
@@ -76,7 +76,7 @@ namespace gloox
         {
           case TRACK_REQUEST_AUTH_FIELDS:
           {
-            std::string id = m_parent->getID();
+            const std::string& id = m_parent->getID();
 
             Tag *iq = new Tag( "iq" );
             iq->addAttribute( "id", id );
@@ -89,15 +89,11 @@ namespace gloox
             Tag *q = stanza->findChild( "query" );
             if( ( q->hasChild( "digest" ) ) && !m_sid.empty() )
             {
-              const std::string pwd = m_parent->password();
-              char buf[41];
-              iksha *sha;
-              sha = iks_sha_new();
-              iks_sha_hash( sha, (const unsigned char*)m_sid.c_str(), m_sid.length(), 0 );
-              iks_sha_hash( sha, (const unsigned char*)pwd.c_str(), pwd.length(), 1 );
-              iks_sha_print( sha, buf );
-              iks_sha_delete( sha );
-              new Tag( query, "digest", buf );
+              SHA sha;
+              sha.feed( m_sid );
+              sha.feed( m_parent->password() );
+              sha.finalize();
+              new Tag( query, "digest", sha.hex() );
             }
             else
             {
