@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -27,13 +27,13 @@ namespace gloox
    * Upon an incoming Presence packet @ref handlePresence() will be called.
    * @author Jakob Schroeter <js@camaya.net>
    */
-  class GLOOX_EXPORT PresenceHandler
+  class GLOOX_API PresenceHandler
   {
     public:
       /**
        * Virtual Destructor.
        */
-      virtual ~PresenceHandler() {};
+      virtual ~PresenceHandler() {}
 
       /**
        * Reimplement this function if you want to be updated on
