@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -20,7 +20,7 @@ namespace gloox
   PrivacyItem::PrivacyItem( const ItemType type, const ItemAction action,
                             const int packetType, const std::string& value )
     : m_type( type ), m_action( action ), m_packetType( packetType ),
-    m_value( value )
+      m_value( value )
   {
   }
 
@@ -28,7 +28,7 @@ namespace gloox
   {
   }
 
-  bool PrivacyItem::operator==( PrivacyItem& item )
+  bool PrivacyItem::operator==( PrivacyItem& item ) const
   {
     if( m_type == item.type()
         && m_action == item.action()
@@ -38,4 +38,5 @@ namespace gloox
     else
       return false;
   }
+
 }
