@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -35,7 +35,7 @@ namespace gloox
   std::string PrivateXML::requestXML( const std::string& tag, const std::string& xmlns,
                                       PrivateXMLHandler *pxh )
   {
-    const std::string id = m_parent->getID();
+    const std::string& id = m_parent->getID();
 
     Tag *iq = new Tag( "iq" );
     iq->addAttribute( "id", id );
@@ -46,7 +46,7 @@ namespace gloox
     x->addAttribute( "xmlns", xmlns );
 
     m_track[id] = pxh;
-    m_parent->trackID( this, id, REQUEST_XML );
+    m_parent->trackID( this, id, RequestXml );
     m_parent->send( iq );
 
     return id;
@@ -54,7 +54,7 @@ namespace gloox
 
   std::string PrivateXML::storeXML( Tag *tag, PrivateXMLHandler *pxh )
   {
-    const std::string id = m_parent->getID();
+    const std::string& id = m_parent->getID();
 
     Tag *iq = new Tag( "iq" );
     iq->addAttribute( "id", id );
@@ -64,7 +64,7 @@ namespace gloox
     query->addChild( tag );
 
     m_track[id] = pxh;
-    m_parent->trackID( this, id, STORE_XML );
+    m_parent->trackID( this, id, StoreXml );
     m_parent->send( iq );
 
     return id;
@@ -81,12 +81,12 @@ namespace gloox
         {
           switch( context )
           {
-            case REQUEST_XML:
+            case RequestXml:
             {
               Tag *q = stanza->findChild( "query" );
               if( q )
               {
-                Tag::TagList l = q->children();
+                const Tag::TagList& l = q->children();
                 Tag::TagList::const_iterator it = l.begin();
                 if( it != l.end() )
                 {
@@ -96,12 +96,13 @@ namespace gloox
               break;
             }
 
-            case STORE_XML:
+            case StoreXml:
             {
-              (*t).second->handlePrivateXMLResult( stanza->id(), PrivateXMLHandler::PXML_STORE_OK );
+              (*t).second->handlePrivateXMLResult( stanza->id(), PrivateXMLHandler::PxmlStoreOk );
               break;
             }
           }
+          m_track.erase( t );
           return true;
           break;
         }
@@ -109,15 +110,15 @@ namespace gloox
         {
           switch( context )
           {
-            case REQUEST_XML:
+            case RequestXml:
             {
-              (*t).second->handlePrivateXMLResult( stanza->id(), PrivateXMLHandler::PXML_REQUEST_ERROR );
+              (*t).second->handlePrivateXMLResult( stanza->id(), PrivateXMLHandler::PxmlRequestError );
               break;
             }
 
-            case STORE_XML:
+            case StoreXml:
             {
-              (*t).second->handlePrivateXMLResult( stanza->id(), PrivateXMLHandler::PXML_STORE_ERROR );
+              (*t).second->handlePrivateXMLResult( stanza->id(), PrivateXMLHandler::PxmlStoreError );
               break;
             }
           }
@@ -129,6 +130,7 @@ namespace gloox
 
       m_track.erase( t );
     }
+
     return false;
   }
 
