@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -22,12 +22,19 @@
 namespace gloox
 {
 
+  class Stanza;
+
+  /**
+   * A map of JID/RosterItem pairs.
+   */
+  typedef std::map<const std::string, RosterItem*> Roster;
+
   /**
    * @brief A virtual interface which can be reimplemented to receive roster updates.
    *
-   * A class implementing this interface and being registered as RosterListener with the Roster
-   * object receives notifications about all the changes in the server-side roster.
-   * Only one RosterListener per Roster at a time is possible.
+   * A class implementing this interface and being registered as RosterListener with the
+   * RosterManager object receives notifications about all the changes in the server-side
+   * roster. Only one RosterListener per Roster at a time is possible.
    *
    * @author Jakob Schroeter <js@camaya.net>
    * @since 0.3
@@ -35,15 +42,10 @@ namespace gloox
   class GLOOX_API RosterListener
   {
     public:
-      /**
-       * A map of JID/RosterItem pairs.
-       */
-      typedef std::map<const std::string, RosterItem*> Roster;
-
       /**
        * Virtual Destructor.
        */
-      virtual ~RosterListener() {};
+      virtual ~RosterListener() {}
 
       /**
        * Reimplement this function if you want to be notified about new items
@@ -53,14 +55,14 @@ namespace gloox
        * presence is received without them being on the roster.
        * @param jid The new item's full address.
        */
-      virtual void itemAdded( const std::string& jid ) = 0;
+      virtual void handleItemAdded( const JID& jid ) = 0;
 
       /**
        * Reimplement this function if you want to be notified about items
        * which authorised subscription.
        * @param jid The authorising item's full address.
        */
-      virtual void itemSubscribed( const std::string& jid ) = 0;
+      virtual void handleItemSubscribed( const JID& jid ) = 0;
 
       /**
        * Reimplement this function if you want to be notified about items that
@@ -69,7 +71,7 @@ namespace gloox
        * another.
        * @param jid The removed item's full address.
        */
-      virtual void itemRemoved( const std::string& jid ) = 0;
+      virtual void handleItemRemoved( const JID& jid ) = 0;
 
       /**
        * Reimplement this function if you want to be notified about items that
@@ -78,17 +80,17 @@ namespace gloox
        * server-side contact list. This can include modifying the item's name, its groups, or the
        * subscription status. These changes are pushed by the server to @b all connected resources.
        * This is why this function will be called if you modify a roster item locally and synchronize
-       * it with teh server.
+       * it with the server.
        * @param jid The modified item's full address.
        */
-      virtual void itemUpdated( const std::string& jid ) = 0;
+      virtual void handleItemUpdated( const JID& jid ) = 0;
 
       /**
        * Reimplement this function if you want to be notified about items which
        * removed subscription authorization.
        * @param jid The item's full address.
        */
-      virtual void itemUnsubscribed( const std::string& jid ) = 0;
+      virtual void handleItemUnsubscribed( const JID& jid ) = 0;
 
       /**
        * Reimplement this function if you want to receive the whole server-side roster
@@ -98,38 +100,34 @@ namespace gloox
        * roster push only happens once per connection.
        * @param roster The full roster.
        */
-      virtual void roster( const Roster& roster ) = 0;
+      virtual void handleRoster( const Roster& roster ) = 0;
 
       /**
        * This function is called on every status change of an item in the roster.
-       * @note This function is not called for status changes from or to Unavailable.
-       * In these cases, @ref itemAvailable() and @ref itemUnavailable() are called,
-       * respectively.
+       * If the presence is of type Unavailable, then the resource has already been
+       * removed from the RosterItem.
        * @param item The roster item.
-       * @param status The item's new status.
+       * @param resource The resource that changed presence.
+       * @param presence The item's new presence.
        * @param msg The status change message.
+       * @since 0.9
        */
-      virtual void presenceUpdated( const RosterItem& item, int status, const std::string& msg ) = 0;
+      virtual void handleRosterPresence( const RosterItem& item, const std::string& resource,
+                                         Presence presence, const std::string& msg ) = 0;
 
       /**
-       * This function is called whenever a roster item comes online (is available).
-       * However, it will not be called for status changes from Away (or any other
-       * status which is not Unavailable) to Available.
-       * @param item A reference to the changed roster item.
+       * This function is called on every status change of a JID that matches the Client's
+       * own JID.
+       * If the presence is of type Unavailable, then the resource has already been
+       * removed from the RosterItem.
+       * @param item The self item.
+       * @param resource The resource that changed presence.
+       * @param presence The item's new presence.
        * @param msg The status change message.
-       * @param from The full JID that became available.
+       * @since 0.9
        */
-      virtual void itemAvailable( const RosterItem& item, const std::string& msg,
-                                  const JID& from ) = 0;
-
-      /**
-       * This function is called whenever a roster item goes offline (is unavailable).
-       * @param item A reference to the changed roster item.
-       * @param msg The status change message.
-       * @param from The full JID that became unavailable.
-       */
-      virtual void itemUnavailable( const RosterItem& item, const std::string& msg,
-                                    const JID& from ) = 0;
+      virtual void handleSelfPresence( const RosterItem& item, const std::string& resource,
+                                       Presence presence, const std::string& msg ) = 0;
 
       /**
        * This function is called when an entity wishes to subscribe to this entity's presence.
@@ -141,7 +139,7 @@ namespace gloox
        * @return Return @b true to allow subscription and subscribe to the remote entity's
        * presence, @b false to ignore the request.
        */
-      virtual bool subscriptionRequest( const std::string& jid, const std::string& msg ) = 0;
+      virtual bool handleSubscriptionRequest( const JID& jid, const std::string& msg ) = 0;
 
       /**
        * This function is called when an entity unsubscribes from this entity's presence.
@@ -153,15 +151,20 @@ namespace gloox
        * @param msg A message sent along with the request.
        * @return Return @b true to unsubscribe from the remote entity, @b false to ignore.
        */
-      virtual bool unsubscriptionRequest( const std::string& jid, const std::string& msg ) = 0;
+      virtual bool handleUnsubscriptionRequest( const JID& jid, const std::string& msg ) = 0;
 
       /**
        * This function is called whenever presence from an entity is received which is not in
        * the roster.
-       * @param jid The entity's full JID.
+       * @param stanza The full presence stanza.
        */
-      virtual void nonrosterPresenceReceived( const JID& jid ) = 0;
+      virtual void handleNonrosterPresence( Stanza* stanza ) = 0;
 
+      /**
+       * This function is called if the server returned an error.
+       * @param stanza The error stanza.
+       */
+      virtual void handleRosterError( Stanza* stanza ) = 0;
   };
 
 }
