@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2004-2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2004-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -26,9 +26,14 @@ namespace gloox
    * Upon an incoming Subscription packet @ref handleSubscription() will be called.
    * @author Jakob Schroeter <js@camaya.net>
    */
-  class GLOOX_EXPORT SubscriptionHandler
+  class GLOOX_API SubscriptionHandler
   {
     public:
+      /**
+       * Virtual destructor.
+       */
+      virtual ~SubscriptionHandler() {}
+
       /**
        * Reimplement this function if you want to be notified about incoming
        * subscriptions/subscription requests.
