@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -14,7 +14,7 @@
 #ifndef TAGHANDLER_H__
 #define TAGHANDLER_H__
 
-#include "stanza.h"
+#include "tag.h"
 
 namespace gloox
 {
@@ -22,23 +22,28 @@ namespace gloox
   /**
    * @brief A virtual interface which can be reimplemented to receive non-XMPP Core stanzas.
    *
-   * Derived classes can be registered as TagHandlers with the Client.
-   * A TagHandler can handle arbitrary elements not defined by RFC 3920, XMPP: Core.
+   * Derived classes can be registered as TagHandlers with the ClientBase.
+   * A TagHandler can handle arbitrary elements not defined by RFC 3920, XMPP Core.
+   *
+   * It can also be used to handle Tags emitted by Parser.
+   *
    * @author Jakob Schroeter <js@camaya.net>
    */
-  class GLOOX_EXPORT TagHandler
+  class GLOOX_API TagHandler
   {
      public:
       /**
        * Virtual Destructor.
        */
-       virtual ~TagHandler() {};
+       virtual ~TagHandler() {}
 
        /**
         * This function is called when a registered XML element arrives.
-        * @param stanza The complete Stanza.
+        * As with every handler in gloox, the Tag is going to be deleted after this function returned.
+        * If you need a copy afterwards, create it using Tag::clone().
+        * @param tag The complete Tag.
         */
-       virtual void handleTag( Stanza *stanza ) = 0;
+       virtual void handleTag( Tag *tag ) = 0;
   };
 
 }
